﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Input;
using WpfOsi.Helper;


namespace WpfOsi
{
    /// <summary>
    /// Interaktionslogik für ShowInstructionWindow.xaml
    /// </summary>
    public partial class ShowInstructionWindow : Window, INotifyPropertyChanged
    {
        private Context.OsiContext _ctx;
        public ShowInstructionWindow()
        {
            InitializeComponent();
        }
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            _ctx = new Context.OsiContext();
            InstructionList = _ctx.Instructions;
            SelectedInstruction = InstructionList.FirstOrDefault();
            DataContext = this;
        }
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            _ctx.Dispose();
        }

        #region Properties
        private ObservableCollection<Model.Instruction> _instructionList;

        public ObservableCollection<Model.Instruction> InstructionList
        {
            get { return _instructionList; }
            set { _instructionList = value; RaisePropertyChanged(); }
        }

        private Model.Instruction _selectedInstruction;

        public Model.Instruction SelectedInstruction
        {
            get { return _selectedInstruction; }
            set { _selectedInstruction = value; RaisePropertyChanged(); }
        }
        #endregion
        #region Commands
        private ICommand _addNewInstructionCommand;

        public ICommand AddNewInstructionCommand
        {
            get { return _addNewInstructionCommand ?? new RelayCommand(AddNewInstructionCommand_Execute, AddNewInstructionCommand_CanExecute); }
            set { _addNewInstructionCommand = value; RaisePropertyChanged(); }
        }

        private bool AddNewInstructionCommand_CanExecute(object obj)
        {
            return true;
        }
        private void AddNewInstructionCommand_Execute(object obj)
        {
            //SPS - Falsch Du willst ja eine neue Insteraction anlegen und keine Productionline
            // Ausserdem ist eine Instruction als solche  als Datencontext zu übergeben Suboptimal da in dieser klasse ja auch keine Podutkionline enthalten sind
            // Diese werden aber für das Füllen des DopDowns benötigt oder? Deshalb muss das in das Window selbst
            //AddInstructionWindow dlg = new AddInstructionWindow() { Owner = this, DataContext = new Model.ProductionLine() };
            AddInstructionWindow dlg = new AddInstructionWindow() { Owner = this };
            if (dlg.ShowDialog() == true)
            {
                //InstructionList.Add((Model.Instruction)dlg.DataContext);
                InstructionList.Add(dlg.CurrentInstruction); //Gespeichert wurd nun im Window. nur der Auflistung hinzufügen, so sparen wir uns ein erneutes laden.
            }
        }
        private ICommand _saveCommand;
        public ICommand SaveCommand
        {
            get { return _saveCommand ?? new RelayCommand(SaveCommand_Execute, SaveCommand_CanExecute); }
            set { _saveCommand = value; RaisePropertyChanged(); }
        }

        private bool SaveCommand_CanExecute(object obj)
        {
            return true;
        }
        private void SaveCommand_Execute(object obj)
        {
            _ctx.SaveChanges();
            DialogResult = true;
        }
        private ICommand _abortCommand;
        public ICommand AbortCommand
        {
            get { return _abortCommand ?? new RelayCommand(AbortCommand_Execute, AbortCommand_CanExecute); }
            set { _abortCommand = value; RaisePropertyChanged(); }
        }
        private bool AbortCommand_CanExecute(object obj)
        {
            return true;
        }
        private void AbortCommand_Execute(object obj)
        {
            DialogResult = false;
        }
        #endregion
        #region INotify Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void RaisePropertyChanged([CallerMemberName] string prop = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(prop));
        }
        #endregion
    }
}
