﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Input;
using WpfOsi.Helper;

namespace WpfOsi
{
    /// <summary>
    /// Interaktionslogik für ShowProductionLineWindow.xaml
    /// </summary>
    public partial class ShowProductionLineWindow : Window, INotifyPropertyChanged
    {
        private Context.OsiContext _ctx;
        public ShowProductionLineWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            _ctx = new Context.OsiContext();
            ProductionLineList = _ctx.ProductionLines;
            SelectedProductionLine = ProductionLineList.FirstOrDefault();
            DataContext = this;
        }
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            _ctx.Dispose();
        }

        private ObservableCollection<Model.ProductionLine> _productionLineList;

        public ObservableCollection<Model.ProductionLine> ProductionLineList
        {
            get { return _productionLineList; }
            set { _productionLineList = value; RaisePropertyChanged(); }
        }

        private Model.ProductionLine _selectedProductionLine;

        public Model.ProductionLine SelectedProductionLine
        {
            get { return _selectedProductionLine; }
            set { _selectedProductionLine = value;RaisePropertyChanged(); }
        }


        #region Commands

        private ICommand _addNewProductionLineCommand;

        public ICommand AddNewProductionLineCommand
        {
            get { return _addNewProductionLineCommand ?? new RelayCommand(AddNewProductionLineCommand_Execute, AddNewProductionLineCommand_CanExecute); }
            set { _addNewProductionLineCommand = value;RaisePropertyChanged(); }
        }

        private bool AddNewProductionLineCommand_CanExecute(object obj)
        {
            return true;
        }
        private void AddNewProductionLineCommand_Execute(object obj)
        {
            AddProductionLineWindow dlg = new AddProductionLineWindow() { Owner = this, DataContext = new Model.ProductionLine() };
            if(dlg.ShowDialog() == true)
            {
                ProductionLineList.Add((Model.ProductionLine)dlg.DataContext);
            }
        }
        private ICommand _saveCommand;
        public ICommand SaveCommand
        {
            get { return _saveCommand ?? new RelayCommand(SaveCommand_Execute,SaveCommand_CanExecute); }
            set { _saveCommand = value;RaisePropertyChanged(); }
        }

        private bool SaveCommand_CanExecute(object obj)
        {
            return true;
        }
        private void SaveCommand_Execute(object obj)
        {
            _ctx.SaveChanges();
            DialogResult = true;
        }

        private ICommand _abortCommand;
        public ICommand AbortCommand
        {
            get { return _abortCommand ?? new RelayCommand(AbortCommand_Execute, AbortCommand_CanExecute); }
            set { _abortCommand = value;RaisePropertyChanged(); }
        }
        private bool AbortCommand_CanExecute(object obj)
        {
            return true;
        }
        private void AbortCommand_Execute(object obj)
        {
            DialogResult = false;
        }
        #endregion

        #region INotify Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void RaisePropertyChanged([CallerMemberName] string prop = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(prop));
        }
        #endregion

    }
}
